package javax.swing.text;

import java.awt.Point;
import java.util.BitSet;
import javax.swing.event.DocumentEvent;
import java.util.Locale;
import java.text.BreakIterator;
import sun.swing.SwingUtilities2;
import javax.swing.UIManager;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.Container;
import java.awt.Color;

public class GlyphView extends View implements TabableView, Cloneable
{
    private byte[] selections;
    int offset;
    int length;
    boolean impliedCR;
    boolean skipWidth;
    TabExpander expander;
    private float minimumSpan;
    private int[] breakSpots;
    int x;
    GlyphView.GlyphPainter painter;
    static GlyphView.GlyphPainter defaultPainter;
    private GlyphView.JustificationInfo justificationInfo;
    
    public GlyphView(Element element) {
        super(element);
        this.selections = null;
        this.minimumSpan = -1.0f;
        this.breakSpots = null;
        this.justificationInfo = null;
        this.offset = 0;
        this.length = 0;
        Element parentElement = element.getParentElement();
        AttributeSet attributes = element.getAttributes();
        this.impliedCR = (attributes != null && attributes.getAttribute("CR") != null && parentElement != null && parentElement.getElementCount() > 1);
        this.skipWidth = element.getName().equals("br");
    }
    
    @Override
    protected final Object clone() {
        Object clone;
        try {
            clone = super.clone();
        }
        catch (CloneNotSupportedException ex) {
            clone = null;
        }
        return clone;
    }
    
    public GlyphView.GlyphPainter getGlyphPainter() {
        return this.painter;
    }
    
    public void setGlyphPainter(GlyphView.GlyphPainter painter) {
        this.painter = painter;
    }
    
    public Segment getText(int n, int n2) {
        Segment sharedSegment = SegmentCache.getSharedSegment();
        try {
            this.getDocument().getText(n, n2 - n, sharedSegment);
        }
        catch (BadLocationException obj) {
            throw new StateInvariantError("GlyphView: Stale view: " + obj);
        }
        return sharedSegment;
    }
    
    public Color getBackground() {
        Document document = this.getDocument();
        if ((document instanceof StyledDocument)) {
            AttributeSet attributes = this.getAttributes();
            if (attributes.isDefined(StyleConstants.Background)) {
                return ((StyledDocument)document).getBackground(attributes);
            }
        }
        return null;
    }
    
    public Color getForeground() {
        Document document = this.getDocument();
        if ((document instanceof StyledDocument)) {
            return ((StyledDocument)document).getForeground(this.getAttributes());
        }
        Container container = this.getContainer();
        if (container != null) {
            return container.getForeground();
        }
        return null;
    }
    
    public Font getFont() {
        Document document = this.getDocument();
        if ((document instanceof StyledDocument)) {
            return ((StyledDocument)document).getFont(this.getAttributes());
        }
        Container container = this.getContainer();
        if (container != null) {
            return container.getFont();
        }
        return null;
    }
    
    public boolean isUnderline() {
        return StyleConstants.isUnderline(this.getAttributes());
    }
    
    public boolean isStrikeThrough() {
        return StyleConstants.isStrikeThrough(this.getAttributes());
    }
    
    public boolean isSubscript() {
        return StyleConstants.isSubscript(this.getAttributes());
    }
    
    public boolean isSuperscript() {
        return StyleConstants.isSuperscript(this.getAttributes());
    }
    
    public TabExpander getTabExpander() {
        return this.expander;
    }
    
    protected void checkPainter() {
        if (this.painter == null) {
            if (GlyphView.defaultPainter == null) {
                String str = "javax.swing.text.GlyphPainter1";
                try {
                    ClassLoader classLoader = this.getClass().getClassLoader();
                    Class<Object> clazz;
                    if (classLoader != null) {
                        clazz = classLoader.loadClass(str);
                    }
                    else {
                        clazz = Class.forName(str);
                    }
                    Object instance = clazz.newInstance();
                    if ((instance instanceof GlyphView.GlyphPainter)) {
                        GlyphView.defaultPainter = (GlyphView.GlyphPainter)instance;
                    }
                }
                catch (Throwable t) {
                    throw new StateInvariantError("GlyphView: Can't load glyph painter: " + str);
                }
            }
            this.setGlyphPainter(GlyphView.defaultPainter.getPainter(this, this.getStartOffset(), this.getEndOffset()));
        }
    }
    
    @Override
    public float getTabbedSpan(float n, TabExpander expander) {
        this.checkPainter();
        TabExpander expander2 = this.expander;
        this.expander = expander;
        if (this.expander != expander2) {
            this.preferenceChanged(null, true, false);
        }
        this.x = (int)n;
        return this.painter.getSpan(this, this.getStartOffset(), this.getEndOffset(), this.expander, n);
    }
    
    @Override
    public float getPartialSpan(int n, int n2) {
        this.checkPainter();
        return this.painter.getSpan(this, n, n2, this.expander, (float)this.x);
    }
    
    @Override
    public int getStartOffset() {
        Element element = this.getElement();
        return (this.length > 0) ? (element.getStartOffset() + this.offset) : element.getStartOffset();
    }
    
    @Override
    public int getEndOffset() {
        Element element = this.getElement();
        return (this.length > 0) ? (element.getStartOffset() + this.offset + this.length) : element.getEndOffset();
    }
    
    private void initSelections(int n, int n2) {
        int n3 = n2 - n + 1;
        if (this.selections == null || n3 > this.selections.length) {
            this.selections = new byte[n3];
            return;
        }
        for (int i = 0; i < n3; this.selections[i++] = 0) {}
        return;
    }
    
    @Override
    public void paint(Graphics graphics, Shape shape) {
        this.checkPainter();
        int n = 0;
        Container container = this.getContainer();
        int startOffset = this.getStartOffset();
        int endOffset = this.getEndOffset();
        Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
        Color background = this.getBackground();
        Color foreground = this.getForeground();
        if (container != null && !container.isEnabled()) {
            foreground = ((container instanceof JTextComponent) ? ((JTextComponent)container).getDisabledTextColor() : UIManager.getColor("textInactiveText"));
        }
        if (background != null) {
            graphics.setColor(background);
            graphics.fillRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
        }
        if ((container instanceof JTextComponent)) {
            JTextComponent textComponent = (JTextComponent)container;
            Highlighter highlighter = textComponent.getHighlighter();
            if ((highlighter instanceof LayeredHighlighter)) {
                ((LayeredHighlighter)highlighter).paintLayeredHighlights(graphics, startOffset, endOffset, shape, textComponent, this);
            }
        }
        if (Utilities.isComposedTextElement(this.getElement())) {
            Utilities.paintComposedText(graphics, shape.getBounds(), this);
            n = 1;
        }
        else if ((container instanceof JTextComponent)) {
            JTextComponent c = (JTextComponent)container;
            Color selectedTextColor = c.getSelectedTextColor();
            if (c.getHighlighter() != null && selectedTextColor != null && !selectedTextColor.equals(foreground)) {
                Highlighter.Highlight[] highlights = c.getHighlighter().getHighlights();
                if (highlights.length != 0) {
                    int n2 = 0;
                    int n3 = 0;
                    for (int i = 0; i < highlights.length; ++i) {
                        Highlighter.Highlight h = highlights[i];
                        int startOffset2 = h.getStartOffset();
                        int endOffset2 = h.getEndOffset();
                        if (startOffset2 <= endOffset && endOffset2 >= startOffset && SwingUtilities2.useSelectedTextColor(h, c)) {
                            if (startOffset2 <= startOffset && endOffset2 >= endOffset) {
                                this.paintTextUsingColor(graphics, shape, selectedTextColor, startOffset, endOffset);
                                n = 1;
                                break;
                            }
                            if (n2 == 0) {
                                this.initSelections(startOffset, endOffset);
                                n2 = 1;
                            }
                            int max = Math.max(startOffset, startOffset2);
                            int min = Math.min(endOffset, endOffset2);
                            this.paintTextUsingColor(graphics, shape, selectedTextColor, max, min);
                            byte[] selections = this.selections;
                            int n4 = max - startOffset;
                            ++selections[n4];
                            byte[] selections2 = this.selections;
                            int n5 = min - startOffset;
                            --selections2[n5];
                            ++n3;
                        }
                    }
                    if (n == 0 && n3 > 0) {
                        int n6 = -1;
                        int n7 = 0;
                        for (int n8 = endOffset - startOffset; n6++ < n8; n7 = n6) {
                            while (n6 < n8 && this.selections[n6] == 0) {
                                ++n6;
                            }
                            if (n7 != n6) {
                                this.paintTextUsingColor(graphics, shape, foreground, startOffset + n7, startOffset + n6);
                            }
                            for (int n9 = 0; n6 < n8 && (n9 += this.selections[n6]) != 0; ++n6) {}
                        }
                        n = 1;
                    }
                }
            }
        }
        if (n == 0) {
            this.paintTextUsingColor(graphics, shape, foreground, startOffset, endOffset);
        }
    }
    
    final void paintTextUsingColor(Graphics graphics, Shape shape, Color color, int n, int n2) {
        graphics.setColor(color);
        this.painter.paint(this, graphics, shape, n, n2);
        int underline = this.isUnderline();
        int strikeThrough = this.isStrikeThrough();
        if (underline || strikeThrough) {
            Rectangle rectangle = (shape instanceof Rectangle) ? ((Rectangle)shape) : shape.getBounds();
            View parent = this.getParent();
            if (parent != null && parent.getEndOffset() == n2) {
                Segment text = this.getText(n, n2);
                while (Character.isWhitespace(text.last())) {
                    --n2;
                    --text.count;
                }
                SegmentCache.releaseSharedSegment(text);
            }
            int x = rectangle.x;
            int startOffset = this.getStartOffset();
            if (startOffset != n) {
                x += (int)this.painter.getSpan(this, startOffset, n, this.getTabExpander(), (float)x);
            }
            int n3 = x + (int)this.painter.getSpan(this, n, n2, this.getTabExpander(), (float)x);
            int n4 = rectangle.y + (int)(this.painter.getHeight(this) - this.painter.getDescent(this));
            if (!(underline == 0)) {
                int n5 = n4 + 1;
                graphics.drawLine(x, n5, n3, n5);
            }
            if (!(strikeThrough == 0)) {
                int n6 = n4 - (int)(this.painter.getAscent(this) * 0.3f);
                graphics.drawLine(x, n6, n3, n6);
            }
        }
    }
    
    @Override
    public float getMinimumSpan(int i) {
        switch (i) {
            case 0: {
                if (this.minimumSpan < 0.0f) {
                    this.minimumSpan = 0.0f;
                    int breakSpot;
                    for (int startOffset = this.getStartOffset(), j = this.getEndOffset(); j > startOffset; this.minimumSpan = Math.max(this.minimumSpan, this.getPartialSpan(breakSpot, j)), j = breakSpot - 1) {
                        breakSpot = this.getBreakSpot(startOffset, j);
                        if (breakSpot == -1) {
                            breakSpot = startOffset;
                        }
                    }
                }
                return this.minimumSpan;
            }
            case 1: {
                return super.getMinimumSpan(i);
            }
            default: {
                throw new IllegalArgumentException("Invalid axis: " + i);
            }
        }
    }
    
    @Override
    public float getPreferredSpan(int i) {
        if (this.impliedCR) {
            return 0.0f;
        }
        this.checkPainter();
        int startOffset = this.getStartOffset();
        int endOffset = this.getEndOffset();
        switch (i) {
            case 0: {
                if (this.skipWidth) {
                    return 0.0f;
                }
                return this.painter.getSpan(this, startOffset, endOffset, this.expander, (float)this.x);
            }
            case 1: {
                float height = this.painter.getHeight(this);
                if (this.isSuperscript()) {
                    height += height / 3.0f;
                }
                return height;
            }
            default: {
                throw new IllegalArgumentException("Invalid axis: " + i);
            }
        }
    }
    
    @Override
    public float getAlignment(int n) {
        this.checkPainter();
        if (n == 1) {
            int superscript = this.isSuperscript();
            int subscript = this.isSubscript();
            float height = this.painter.getHeight(this);
            float descent = this.painter.getDescent(this);
            float ascent = this.painter.getAscent(this);
            float n2;
            if (!(superscript == 0)) {
                n2 = 1.0f;
            }
            else if (!(subscript == 0)) {
                n2 = ((height > 0.0f) ? ((height - (descent + ascent / 2.0f)) / height) : 0.0f);
            }
            else {
                n2 = ((height > 0.0f) ? ((height - descent) / height) : 0.0f);
            }
            return n2;
        }
        else {
            return super.getAlignment(n);
        }
    }
    
    @Override
    public Shape modelToView(int n, Shape shape, Position.Bias bias) throws BadLocationException {
        this.checkPainter();
        return this.painter.modelToView(this, n, bias, shape);
    }
    
    @Override
    public int viewToModel(float n, float n2, Shape shape, Position.Bias[] array) {
        this.checkPainter();
        return this.painter.viewToModel(this, n, n2, shape, array);
    }
    
    @Override
    public int getBreakWeight(int n, float n2, float n3) {
        if (n == 0) {
            this.checkPainter();
            int startOffset = this.getStartOffset();
            int boundedPosition = this.painter.getBoundedPosition(this, startOffset, n2, n3);
            return (boundedPosition == startOffset) ? 0 : ((this.getBreakSpot(startOffset, boundedPosition) != -1) ? 2000 : 1000);
        }
        return super.getBreakWeight(n, n2, n3);
    }
    
    @Override
    public View breakView(int n, int n2, float n3, float n4) {
        if (n == 0) {
            this.checkPainter();
            int boundedPosition = this.painter.getBoundedPosition(this, n2, n3, n4);
            int breakSpot = this.getBreakSpot(n2, boundedPosition);
            if (breakSpot != -1) {
                boundedPosition = breakSpot;
            }
            if (n2 == this.getStartOffset() && boundedPosition == this.getEndOffset()) {
                return this;
            }
            GlyphView glyphView = (GlyphView)this.createFragment(n2, boundedPosition);
            glyphView.x = (int)n3;
            return glyphView;
        }
        else {
            return this;
        }
    }
    
    private int getBreakSpot(int n, int n2) {
        if (this.breakSpots == null) {
            int startOffset = this.getStartOffset();
            int endOffset = this.getEndOffset();
            int[] array = new int[endOffset + 1 - startOffset];
            int n3 = 0;
            Element parentElement = this.getElement().getParentElement();
            int n4 = (parentElement == null) ? startOffset : parentElement.getStartOffset();
            int n5 = (parentElement == null) ? endOffset : parentElement.getEndOffset();
            Segment text = this.getText(n4, n5);
            text.first();
            BreakIterator breaker = this.getBreaker();
            breaker.setText(text);
            int n6 = endOffset + ((n5 > endOffset) ? 1 : 0);
            while (true) {
                n6 = breaker.preceding(text.offset + (n6 - n4)) + (n4 - text.offset);
                if (n6 > startOffset) {
                    array[n3++] = n6;
                }
                else {
                    break;
                }
            }
            SegmentCache.releaseSharedSegment(text);
            System.arraycopy(array, 0, this.breakSpots = new int[n3], 0, n3);
        }
        int n7 = -1;
        int i = 0;
        while (i < this.breakSpots.length) {
            int n8 = this.breakSpots[i];
            if (n8 <= n2) {
                if (n8 > n) {
                    n7 = n8;
                    break;
                }
                break;
            }
            else {
                ++i;
            }
        }
        return n7;
    }
    
    private BreakIterator getBreaker() {
        Document document = this.getDocument();
        if (document != null && Boolean.TRUE.equals(document.getProperty(AbstractDocument.MultiByteProperty))) {
            Container container = this.getContainer();
            return BreakIterator.getLineInstance((container == null) ? Locale.getDefault() : container.getLocale());
        }
        return new WhitespaceBasedBreakIterator();
    }
    
    @Override
    public View createFragment(int n, int n2) {
        this.checkPainter();
        Element element = this.getElement();
        GlyphView glyphView = (GlyphView)this.clone();
        glyphView.offset = n - element.getStartOffset();
        glyphView.length = n2 - n;
        glyphView.painter = this.painter.getPainter(glyphView, n, n2);
        glyphView.justificationInfo = null;
        return glyphView;
    }
    
    @Override
    public int getNextVisualPositionFrom(int n, Position.Bias bias, Shape shape, int n2, Position.Bias[] array) throws BadLocationException {
        if (n < -1) {
            throw new BadLocationException("invalid position", n);
        }
        return this.painter.getNextVisualPositionFrom(this, n, bias, shape, n2, array);
    }
    
    @Override
    public void insertUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.justificationInfo = null;
        this.breakSpots = null;
        this.minimumSpan = -1.0f;
        this.syncCR();
        this.preferenceChanged(null, true, false);
    }
    
    @Override
    public void removeUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.justificationInfo = null;
        this.breakSpots = null;
        this.minimumSpan = -1.0f;
        this.syncCR();
        this.preferenceChanged(null, true, false);
    }
    
    @Override
    public void changedUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.minimumSpan = -1.0f;
        this.syncCR();
        this.preferenceChanged(null, true, true);
    }
    
    private void syncCR() {
        if (this.impliedCR) {
            Element parentElement = this.getElement().getParentElement();
            this.impliedCR = (parentElement != null && parentElement.getElementCount() > 1);
        }
    }
    
    @Override
    void updateAfterChange() {
        this.breakSpots = null;
    }
    
    GlyphView.JustificationInfo getJustificationInfo(int n) {
        if (this.justificationInfo != null) {
            return this.justificationInfo;
        }
        int startOffset = this.getStartOffset();
        int endOffset = this.getEndOffset();
        Segment text = this.getText(startOffset, endOffset);
        int offset = text.offset;
        int i = text.offset + text.count - 1;
        int n2 = i + 1;
        int n3 = offset - 1;
        int n4 = 0;
        int n5 = 0;
        int n6 = 0;
        boolean n7 = false;
        BitSet set = new BitSet(endOffset - startOffset + 1);
        int n8 = 0;
        while (i >= offset) {
            if (' ' == text.array[i]) {
                set.set(i - offset);
                if (n8 == 0) {
                    ++n4;
                }
                else if (n8 == 1) {
                    n8 = 2;
                    n6 = 1;
                }
                else if (n8 == 2) {
                    ++n6;
                }
            }
            else {
                if ('\t' == text.array[i]) {
                    n7 = true;
                    break;
                }
                if (n8 == 0) {
                    if ('\n' != text.array[i] && '\r' != text.array[i]) {
                        n8 = 1;
                        n3 = i;
                    }
                }
                else if (n8 != 1 && n8 == 2) {
                    n5 += n6;
                    n6 = 0;
                }
                n2 = i;
            }
            --i;
        }
        SegmentCache.releaseSharedSegment(text);
        int n9 = -1;
        if (n2 < i) {
            n9 = n2 - offset;
        }
        int n10 = -1;
        if (n3 > offset) {
            n10 = n3 - offset;
        }
        return this.justificationInfo = new GlyphView.JustificationInfo(n9, n10, n6, n5, n4, (boolean)n7, set);
    }
    
    static class JustificationInfo
    {
        final int start;
        final int end;
        final int leadingSpaces;
        final int contentSpaces;
        final int trailingSpaces;
        final boolean hasTab;
        final BitSet spaceMap;
        
        JustificationInfo(int start, int end, int leadingSpaces, int contentSpaces, int trailingSpaces, boolean hasTab, BitSet spaceMap) {
            this.start = start;
            this.end = end;
            this.leadingSpaces = leadingSpaces;
            this.contentSpaces = contentSpaces;
            this.trailingSpaces = trailingSpaces;
            this.hasTab = hasTab;
            this.spaceMap = spaceMap;
        }
    }
    
    public abstract static class GlyphPainter
    {
        public abstract float getSpan(GlyphView p0, int p1, int p2, TabExpander p3, float p4);
        
        public abstract float getHeight(GlyphView p0);
        
        public abstract float getAscent(GlyphView p0);
        
        public abstract float getDescent(GlyphView p0);
        
        public abstract void paint(GlyphView p0, Graphics p1, Shape p2, int p3, int p4);
        
        public abstract Shape modelToView(GlyphView p0, int p1, Position.Bias p2, Shape p3) throws BadLocationException;
        
        public abstract int viewToModel(GlyphView p0, float p1, float p2, Shape p3, Position.Bias[] p4);
        
        public abstract int getBoundedPosition(GlyphView p0, int p1, float p2, float p3);
        
        public GlyphView.GlyphPainter getPainter(GlyphView glyphView, int n, int n2) {
            return this;
        }
        
        public int getNextVisualPositionFrom(GlyphView glyphView, int n, Position.Bias bias, Shape shape, int i, Position.Bias[] array) throws BadLocationException {
            int startOffset = glyphView.getStartOffset();
            int endOffset = glyphView.getEndOffset();
            switch (i) {
                case 1:
                case 5: {
                    if (n != -1) {
                        return -1;
                    }
                    Container container = glyphView.getContainer();
                    if ((container instanceof JTextComponent)) {
                        Caret caret = ((JTextComponent)container).getCaret();
                        Point point = (caret != null) ? caret.getMagicCaretPosition() : null;
                        if (point == null) {
                            array[0] = Position.Bias.Forward;
                            return startOffset;
                        }
                        return glyphView.viewToModel((float)point.x, 0.0f, shape, array);
                    }
                    else {
                        return n;
                    }
                }
                case 3: {
                    if (startOffset == glyphView.getDocument().getLength()) {
                        if (n == -1) {
                            array[0] = Position.Bias.Forward;
                            return startOffset;
                        }
                        return -1;
                    }
                    else {
                        if (n == -1) {
                            array[0] = Position.Bias.Forward;
                            return startOffset;
                        }
                        if (n == endOffset) {
                            return -1;
                        }
                        if (++n == endOffset) {
                            return -1;
                        }
                        array[0] = Position.Bias.Forward;
                        return n;
                    }
                }
                case 7: {
                    if (startOffset == glyphView.getDocument().getLength()) {
                        if (n == -1) {
                            array[0] = Position.Bias.Forward;
                            return startOffset;
                        }
                        return -1;
                    }
                    else {
                        if (n == -1) {
                            array[0] = Position.Bias.Forward;
                            return endOffset - 1;
                        }
                        if (n == startOffset) {
                            return -1;
                        }
                        array[0] = Position.Bias.Forward;
                        return n - 1;
                    }
                }
                default: {
                    throw new IllegalArgumentException("Bad direction: " + i);
                }
            }
        }
    }
}
